function varargout = fisher(varargin)
% VL_FISHER    Fisher vector feature encoding
%   ENC = VL_FISHER(X, MEANS, COVARIANCES, PRIORS) computes the
%   Fisher vector encoding of the vectors X relative to the Gaussian
%   mixture model with means MEANS, covariances COVARIANCES, and pror
%   mode probabilities PRIORS.
%
%   X has one column per data vector (e.g. a SIFT descriptor), and
%   MEANS and COVARIANCES one column per GMM component (covariance
%   matrices are assumed diagonal). PRIORS has size equal to the
%   number of GMM components. All data must be of the smae class,
%   either SINGLE or DOUBLE.
%
%   ENC is a vector of the same class of X of size equal to the
%   product of the data dimension and the number of components.
%
%   By default, the standard Fisher vector is computed. VL_FISER()
%   accepts the following options:
%
%   Normalized::
%     If specified, L2 normalize the Fisher vector.
%
%   SquareRoot::
%     If specified, the signed square root function is applied to
%     ENC before normalization.
%
%   Verbose::
%     Increase the verbosity level (may be specified multiple times).
%
%   See: http://www.vlfeat.org/doc/api/fisher.html, VL_HELP().
[varargout{1:nargout}] = vl_fisher(varargin{:});
